@@ -557,9 +557,9 @@
                 :test-selectors {:default (with-meta '(constantly true)
                                             {:displace true})}
                 ;; bump deps in leiningen's own project.clj with these
-                :dependencies '[^:displace [nrepl/nrepl "0.6.0"
+                :dependencies '[^:displace [nrepl/nrepl "0.8.3"
                                             :exclusions [org.clojure/clojure]]
-                                ^:displace [clojure-complete "0.2.5"
+                                ^:displace [org.nrepl/incomplete "0.1.0"
                                             :exclusions [org.clojure/clojure]]]
                 :checkout-deps-shares [:source-paths
                                        :test-paths
@@ -583,7 +583,7 @@
    :provided {:pom-scope :provided}
    :repl {:repl true}})
 
-(defn- meta-merge
+(defn meta-merge
   "Recursively merge values based on the information in their metadata."
   [left right]
   (cond (different-priority? left right)
@@ -611,7 +611,7 @@
         (= (class left) (class right)) right
 
         :else
-        (do (warn left "and" right "have a type mismatch merging profiles.")
+        (do (warn-once left "and" right "have a type mismatch merging profiles.")
             right)))
 
 (defn- apply-profiles [project profiles]
@@ -632,7 +632,10 @@
           (when-not (or result (#{:provided :dev :user :test :base :default
                                   :production :system :repl}
                                 profile))
-            (warn "Warning: profile" profile "not found."))
+            (warn-once "Warning: profile" profile "not found."))
+          (when (and (= :provided profile) (composite-profile? result))
+            (throw (Exception.
+                    "Composite profiles are incompatible with :provided.")))
           (lookup-profile* profiles result))
 
         (composite-profile? profile)
@@ -1120,8 +1123,8 @@ Also initializes the project; see read-raw for a version that skips init."
       (try (read project)
            (catch Exception e
              (throw (Exception. (format "Problem loading %s" project) e)))))
-    (warn "WARN ignoring checkouts directory" (.getParent project-file)
-             "as it does not contain a project.clj file.")))
+    (warn-once "WARNING: ignoring checkouts directory" (.getParent project-file)
+               "as it does not contain a project.clj file.")))
 
 (alter-var-root #'read-dependency-project memoize)
 
